{% extends "base.html" %}

{% block content %}
<h2>All tags ({{ tags | length }})</h2>
<ul class="post-tags all-tags">
{% for tag in tags %}
  <a href="/tags/{{ tag }}"><span class="post-tag">#{{ tag }}</span></a>
{% endfor %}
</ul>
{% endblock %}
